package protocolP2P;
import exception.ProtocolError;
import exception.VersionError;
import protocolP2P.Payload;
import protocolP2P.RequestResponseCode;
import java.util.ArrayList;
import java.lang.Byte;

/** Representation of datagram.
 * @author	Louis Royer
 * @author	Flavien Haas
 * @author	JS Auge
 * @version	1.0
 */
public class ProtocolP2PDatagram {
	private final static byte PROTOCOL_VERSION = 0x11;
	private final static int VERSION_POSITON = 0;
	private byte version;
	private Payload payload;

	/** Constructor with payload parameter (typically used when sending datagram).
	 * @param payload	the payload associated with the datagram to send
	 */
	public ProtocolP2PDatagram(Payload payload) {
		version = PROTOCOL_VERSION;
		this.payload = payload;
	}

	/** Constructor with datagram as byte[] parameter (typically used when receiving datagram).
	 * @param datagram	the full datagram received
	 * @throws	ProtocolError
	 * @throws	VersionError
	 */
	public ProtocolP2PDatagram(byte[] datagram) throws ProtocolError, VersionError {
		// unwrap version
		version = datagram[VERSION_POSITON];
		checkProtocolVersion(); // this can throw VersionError
		RequestResponseCode r = RequestResponseCode.fromCode(datagram[RequestResponseCode.RRCODE_POSITION]); // this can throw ProtocolError
		switch (r) {
			case RequestResponseCode.LIST_RESPONSE:
				payload = (Payload) new FileList(datagram);
				break;
			case RequestResponseCode.LOAD_RESPONSE:
				payload = (Payload) new FilePart(datagram);
				break;
			default:
				payload = new Payload(datagram);
				break;
			}
	}

	/** Returns a byte[] containing full datagram (typically used when sending datagram).
	 * This datagram is still complete and ready to be send.
	 * @return the full datagram to send
	 */
	public byte[] toDatagram() {
		byte[] datagram = payload.toDatagram();
		datagram[VERSION_POSITON] = version;
		return datagram;
	}

	/** Returns Payload associated with the datagram.
	 * @return	payload associated with the datagram
	 */
	public Payload getPayload() {
		return payload;
	}

	/** Used to check protocol version when a datagram is constructed from bytes[].
	 * @throws	VersionError
	 */
	private void checkProtocolVersion() throws VersionError {
		if (PROTOCOL_VERSION != version) {
			throw new VersionError();
		}
	}
}

